<template>
  <input
    :id="id"
    v-model="modelValue"
    :autocomplete="autocomplete"
    :class="classes"
    :name="name"
    :placeholder="placeholder"
    :required="required"
    :type="type"
    class="block w-full rounded-md border-0 py-1.5 text-sm text-gray-900 ring-1 ring-inset placeholder:text-gray-400 focus:outline-none focus:ring-2 focus:ring-primary-500 sm:leading-6"
    v-bind="attrs"
  />
</template>

<script generic="T" lang="ts" setup>
import { computed, type ComputedRef } from "vue";
import { isUndefined, omitBy } from "lodash-es";

const props = withDefaults(
  defineProps<{
    type?: "text" | "email" | "password" | "number" | "tel" | "url";
    name: string;
    required?: boolean;
    placeholder?: string;
    autocomplete?: string;
  }>(),
  {
    type: "text",
    placeholder: undefined,
    autocomplete: undefined,
  },
);

const modelValue = defineModel<T>();

const id = inject<string | undefined>("id", undefined);
const errors = inject<ComputedRef<boolean> | undefined>("errors", undefined);

const attrs = computed(() =>
  omitBy(
    {
      "data-1p-ignore": props.autocomplete === "off" ? true : undefined,
      "data-lpignore": props.autocomplete === "off" ? true : undefined,
    },
    isUndefined,
  ),
);

const classes = computed(() => ({
  "ring-gray-900/10": !errors?.value,
  "text-red-600 ring-red-500 placeholder:text-red-400 focus:ring-red-600": errors?.value,
}));
</script>
